var Player;
function VideoPlayer(plugin, fsRect, wmRect) {
  this.STOPPED = 'stopped';
  this.PLAYING = 'playing';
  this.PAUSED = 'paused';
  
  this.FS = 1; //fullscreen;
  this.WM = 0; //window;

  this.plugin = plugin;

  this.screenRects = {};
  this.screenRects[this.FS] = fsRect;
  this.screenRects[this.WM] = wmRect || fsRect;
  this.setScreenMode(this.WM);

  Player = this;
  // actions handlers
  this.plugin.OnCurrentPlayTime = 'Player.onCurrentPlayTime';
  this.plugin.OnStreamInfoReady = 'Player.onStreamInfoReady';
  this.plugin.OnBufferingStart = 'Player.onBufferingStart';
  this.plugin.OnBufferingProgress = 'Player.onBufferingProgress';
  this.plugin.OnBufferingComplete = 'Player.onBufferingComplete';
  this.plugin.OnRenderingComplete = 'Player.onRenderingComplete';
  // errors handlers
  this.plugin.OnConnectionFailed = 'Player.onConnectionFailed';
  this.plugin.OnStreamNotFound = 'Player.onStreamNotFound';
  this.plugin.OnAuthenticationFailed = 'Player.onAuthenticationFailed';
  this.plugin.OnRenderError = 'Player.onRenderError';
  this.plugin.OnNetworkDisconnected = 'Player.onNetworkDisconnected';
}

///// actions with 'stream' /////

// public
VideoPlayer.prototype.play = function(urlConfig) {
  urlConfig && this.setUrl(urlConfig);

  this.isState(this.PLAYING) && this.stop();
  var success;
  if (this.isState(this.PAUSED)) {
    success = this.plugin.Resume();
  } else {
    var c = this.urlConfig;
    if (c.continue && c.watched_percents) {
      this.urlConfig.continue = false;
      alert("play with offset " + Math.floor(c.watched_percents * c.duration / 100) );
      success = this.plugin.ResumePlay(this.getUrl(), Math.floor(c.watched_percents * c.duration / 100));
    } else {
      success = this.plugin.Play(this.getUrl());
    }
  }
  success && this.setState(this.PLAYING);

  //because of samsung sets FS rect after start playing
  this.setVideoRect(this.screenRects[this.screenMode]);
  alert("VideoPlayer.play() " + success + " url = " + this.url);
  return success;
}

// public
VideoPlayer.prototype.pause = function() {
  this.isState(this.PLAYING) && this.plugin.Pause();
  this.setState(this.PAUSED);
}

// public
VideoPlayer.prototype.togglePlay = function() {
  this.isState(this.PLAYING) ? this.pause() : this.play();
}

// public
VideoPlayer.prototype.stop = function() {
  !this.isState(this.STOPPED) && this.plugin.Stop();
  this.setState(this.STOPPED);
  this.clearScreen();
  this.setSeekEnabled(false);
}

// public
VideoPlayer.prototype.fw = function(offset, seekReadyCallback) {
  if (this.getSeekEnabled()) {
    this.seekReadyCallback = seekReadyCallback;
    fireEvent(this, Events.PLAYER_SEEK);
    this.plugin.JumpForward(offset || Consts.PLAYER_SEEK_OFFSET);
  }
}

// public
VideoPlayer.prototype.bw = function(offset, seekReadyCallback) {
  if (this.getSeekEnabled()) {
    this.seekReadyCallback = seekReadyCallback;
    fireEvent(this, Events.PLAYER_SEEK);
    this.plugin.JumpBackward(offset || Consts.PLAYER_SEEK_OFFSET);
  }
}

// public
VideoPlayer.prototype.reconnect = function() {
  alert("VideoPlayer.reconnect()");
  var self = this;
  var playAttempt = function() {
    self.urlIndex = ++self.urlIndex % self.urlConfig.streamUrls.length;
    if (self.play()) {
      clearInterval(self.reconnectAttemptTimer);
    }
  }
  self.reconnectAttemptTimer = setInterval( playAttempt, 10 * 1000);
}

///// actions with screen //////////

// public
VideoPlayer.prototype.setScreenMode = function(mode) {
  if (mode != this.screenMode) {
    this.screenMode = mode;
    this.setVideoRect(this.screenRects[this.screenMode]);
    fireEvent(this, Events.PLAYER_SCREEN_MODE_CHANGED, this.screenMode);
  }
}

// public
VideoPlayer.prototype.getScreenMode = function() {
  return this.screenMode;
}

// public
VideoPlayer.prototype.toggleFS = function() {
  this.setScreenMode((this.screenMode + 1) % 2);
}

// public
VideoPlayer.prototype.setScreenModeRect = function(mode, rect) {
  this.screenRects[mode] = rect;
  (mode == this.screenMode) && this.setVideoRect(rect);
}

// public
VideoPlayer.prototype.clearScreen = function() {
  this.plugin.ClearScreen();
}

////////////////////////////////////////////////

// public
VideoPlayer.prototype.setUrl = function(urlConfig) {
  this.urlIndex = 0;
  this.urlConfig = urlConfig;
}

// public
VideoPlayer.prototype.isState = function(state) {
  return state == this.state;
}

/////////////////////////////////////////////////////

// private
VideoPlayer.prototype.getUrl = function() {
  var url = null;
  if (this.urlConfig && this.urlConfig.streamUrls) {
    url = this.urlConfig.streamUrls[this.urlIndex] + '|COMPONENT=HLS'; // TODO: check if exists HLS
  }
  return url;
}

// private
VideoPlayer.prototype.setSeekEnabled = function(seekEnabled) {
  this.seekEnabled = seekEnabled;
  fireEvent(this, Events.PLAYER_SEEK_ENABLED_CHANGED, this.seekEnabled);
}

// private
VideoPlayer.prototype.getSeekEnabled = function() {
  return this.seekEnabled;
}

// private
VideoPlayer.prototype.setState = function(state) {
  if(this.state != state) {
    this.state = state;
    alert("VideoPlayer.setState() " + state);
    if (this.PLAYING == this.state) {
      pluginAPI.setOffScreenSaver();
    } else {
      pluginAPI.setOnScreenSaver();
    }
    fireEvent(this, Events.PLAYER_STATE_CHANGED, this.state);
  }
}

// private
VideoPlayer.prototype.setVideoRect = function(rect) {
  this.plugin.SetDisplayArea(rect.x, rect.y, rect.w, rect.h);
  //TODO: event
}


//////////////////////////////////////////////////////////////
VideoPlayer.prototype.onBufferingStart = function() { fireEvent(this, Events.PLAYER_BUFFERING_START); }

VideoPlayer.prototype.onBufferingProgress = function(percent) { fireEvent(this, Events.PLAYER_BUFFERING_PROGRESS, percent);  }

VideoPlayer.prototype.onBufferingComplete = function() {
  if (this.seekReadyCallback) {
    this.seekReadyCallback.fn.call(this.seekReadyCallback.scope);
    this.seekReadyCallback = null;
  }
  fireEvent(this, Events.PLAYER_BUFFERING_COMPLETE);
}

VideoPlayer.prototype.onCurrentPlayTime = function(time) { $(this).trigger( Events.PLAYER_CURRENT_TIME, time); }

VideoPlayer.prototype.onStreamInfoReady = function() {
  var duration = this.plugin.GetDuration();
  if (Consts.PLAYER_LIVE_DURATION == duration || !(this.urlConfig.duration || (this.urlConfig.beginning && this.urlConfig.ending))){
    duration = 0;
  }
  this.setSeekEnabled(0 < duration);
  fireEvent(this, Events.PLAYER_STREAM_INFO_READY, {duration: duration, seekable: this.getSeekEnabled(), isLive: (0 == duration)});
}

VideoPlayer.prototype.onRenderingComplete = function() {
  this.stop();
  fireEvent(this, Events.PLAYER_COMPLETE);
}

//TODO: create error type and send error code
VideoPlayer.prototype.onConnectionFailed = function() { fireEvent(this, Events.PLAYER_ERROR); }

VideoPlayer.prototype.onNetworkDisconnected = function() {
  this.plugin.Stop();
  $(this).trigger(Events.PLAYER_ERROR, 'disconnect');
}

VideoPlayer.prototype.onRenderError = function() { fireEvent(this, Events.PLAYER_ERROR); }

VideoPlayer.prototype.onStreamNotFound = function() { fireEvent(this, Events.PLAYER_ERROR); }

VideoPlayer.prototype.onAuthenticationFailed = function() { fireEvent(this, Events.PLAYER_ERROR); }
