var WidgetsFactory = {
  setScreenShownCallback: function(callback) {
    this.screenShownCallback = callback;
  },

  setDialogShownCallback: function(callback) {
    this.dialogShownCallback = callback;
  },

  setDialogHideCallback:  function(callback) {
    this.dialogHideCallback = callback;
  },

  screens: {},
  getScreen: function(screenType, config) {
    if (!this.screens[screenType]) {
      var screenName = screenType + "Screen";
      var c = $.extend( {
        shownCallback: this.screenShownCallback,
        parentScreen: null,
        rootEl: getElById("ScreenWrapper"),
        mode: 'screen'
      }, config);
      if (window[screenName]) {
        var screen = new window[screenName](c);
        this.screens[screenType] = screen;
      }
    } else {
      this.screens[screenType].updateConfig(config);
    }

    return this.screens[screenType];
  },

  dialogId: 0,

  getDialog: function(type, config) {
    return this.getScreen(type, $.extend({
      shownCallback: this.dialogShownCallback,
      hideCallback: this.dialogHideCallback,
      rootEl: getElById("DialogWrapper"),
      mode: 'dialog',
      dialogId: this.dialogId++
    }, config));
  },

  getNotificationScreen: function(config) {
    return this.getDialog('Notification', config);
  },

  getErrorScreen: function(config) {
    return this.getDialog('Error', config);
  },

  // TODO: fix it. is it screen or dialog?
  getLoginErrorScreen: function(config) {
    var c = $.extend({
        shownCallback: this.dialogShownCallback,
        hideCallback: this.dialogHideCallback,
        rootEl: getElById("DialogWrapper"),
        mode: 'dialog'
      }, config);
    return new LoginErrorScreen(c);
  },

  getWaitingScreen: function(config) {
    return this.getDialog(ScreenTypes.WAITING, config);
  },

  channelsScreen: {
    getChannelsPagedDisplay: function(config) {
      if (!this.channelsPagedDisplay) {
        var rowLength = 3; var rowsCount = 2;
        return new ChannelsPagedDisplay(config.rootEl
        , $.extend(true, {rowLength: rowLength, rowsCount: rowsCount, pageSize: rowLength * rowsCount, hShift: true}
          , config));
      } else {
        this.channelsPagedDisplay.updateConfig(config);
      }
      return this.channelsPagedDisplay;
    }
  },

  liveScreen: {
    getIssuesPagedDisplay: function(config) {
      // TODO: fix it. it needs to be correct with one time creation
     // if (!this.liveIssuesPagedDisplay) {
        var rowsCount = 11;
        this.liveIssuesPagedDisplay = new LiveIssuesPagedDisplay(getElById("LiveScreen.Issues"),
        $.extend(true, {rowsCount: rowsCount, pageSize: rowsCount},
        config));
      //} else {
        //this.liveIssuesPagedDisplay.updateConfig(config);
      //}
      return this.liveIssuesPagedDisplay;
    },

    getChannelMenuNavi: function() {
      var items = [getElById("LiveScreen.ChannelMenu.showLive"), getElById("LiveScreen.ChannelMenu.showTimeshift"),
        getElById("LiveScreen.ChannelMenu.toggleFavorite"), getElById("LiveScreen.ChannelMenu.return")];
      var navi = new Navigable();
      navi.setupNaviWithLine(items, 'v');
      return navi;
    },

    updateFavouritedStatus: function(favourited) {
      $(getElById("LiveScreen.ChannelMenu.toggleFavorite")).html(((favourited) ? Locale.t('Live.ChannelMenu.RemoveFromFav') : Locale.t('Live.ChannelMenu.AddToFav')) + '<i></i>');
    }
  },

  queueScreen: {
    getFilmsPagedDisplay: function(config) {
      if (!this.filmsPagedDisplay) {
        var rowLength = 3; var rowsCount = 2;
        return new FilmsPagedDisplay(config.rootEl
        , $.extend({rowLength: rowLength, rowsCount: rowsCount, pageSize: rowLength * rowsCount, hShift: true}
          , config));
      } else {
        this.filmsPagedDisplay.updateConfig(config);
      }
      return this.filmsPagedDisplay;
    }
  },

  filmScreen: {
    getMenuNavi: function() {
      var items = [getElById("FilmScreen.Menu.showBeginning"),
        getElById("FilmScreen.Menu.removeFromQueue"), getElById("FilmScreen.Menu.return")];
      var navi = new Navigable();
      navi.setupNaviWithLine(items, 'v');
      return navi;
    }
  },

  getPlayerScreen: function(config) {
    return this.getScreen("Player", $.extend({rootEl:getElById("PlayerDisplay"),
      player: AppFactory.getPlayer(), audio: AppFactory.getAudio(), mode: 'player'}, config));
  },

  playerDisplay: {
    getProgressDisplay: function(videoDuration) {
      return ('flash' == AppInfo.videoPluginType() && 0 == videoDuration) ?
        this.getFlashLiveProgressDisplay() :
        this.getStandartProgressDisplay();
    },

    getFlashLiveProgressDisplay: function() {
      var config = {timeStatus: this.timeStatus()};
      if (!this.flashLiveProgressDisplay) {
        this.flashLiveProgressDisplay = new FlashLiveProgressDisplay(config);
      }
      this.flashLiveProgressDisplay.update(config);
      return this.flashLiveProgressDisplay;
    },

    getStandartProgressDisplay: function() {
      var config = {timeStatus: this.timeStatus(), progressStatus: this.progressStatus(), progressBar: this.progressBar(), watchedMark: this.watchedMark(), PROGRESS_BAR_WIDTH: 800};
      if (!this.standartProgressDisplay) {
        this.standartProgressDisplay = new ProgressDisplay(config);
      }
      this.standartProgressDisplay.update(config);
      return this.standartProgressDisplay;
    },

    status: function() {
      return getElById("PlayerDisplay.Status");
    },

    bufferingBar: function() {
      return getElById("PlayerDisplay.BufferingBar");
    },

    progressBar: function() {
      return getElById("PlayerDisplay.ProgressBar");
    },

    progressStatus: function() {
      return getElById("PlayerDisplay.ProgressStatus");
    },

    btTogglePlay: function() {
      var stopped = 0; var playing = 1;
      return new ToggleButton(getElById("PlayerDisplay.Panel.BtTogglePlay"), {
          styles:["player-display-bt-play", "player-display-bt-pause"]
        , states: [stopped, playing]
        , STOPPED: stopped
        , PLAYING: playing
      });
    },

    btBW: function() {
      return getElById("PlayerDisplay.Panel.BtBW");
    },

    btFW: function() {
      return getElById("PlayerDisplay.Panel.BtFW");
    },

    btBack: function() {
      return getElById("PlayerDisplay.Panel.BtBack");
    },

    btToggleMute: function() {
      var volumed = 0; var muted = 1;
      return new ToggleButton(getElById("PlayerDisplay.Panel.BtToggleMute"), {
          styles:["player-display-bt-mute", "player-display-bt-unmute"]
        , states: [volumed, muted]
        , VOLUMED: volumed
        , MUTED: muted
      });
    },

    volumeStatus: function() {
      return getElById("PlayerDisplay.Panel.VolumeStatus");
    },

    timeStatus: function() {
      return getElById("PlayerDisplay.Panel.TimeStatus");
    },

    title: function() {
      return getElById("PlayerDisplay.Title");
    },

    watchedMark: function() {
      return getElById("PlayerDisplay.WatchedMark");
    }
  },

  getToggleFSLayout: function() {
    return getElById("ToggleFS");
  },

  getMainMenu: function() {
    if (!this.mainMenuComp) {
      this.mainMenuComp = new MainMenu(getElById("MainMenu"));
    }
    return this.mainMenuComp;
  },

  mainMenu: {
    getItemsNavi: function() {
      var items = [ getElById("MainMenu.Channels"), getElById("MainMenu.Queue"), getElById("MainMenu.Account")];
      var navi = new Navigable();
      navi.setupNaviWithLine(items, 'h', true);
      return navi;
    },

    getSubItemsNavi: function(itemName) {
      var items = null;
      if ('Channels' == itemName) {
        items = [getElById("MainMenu.Channels.Favorites"), getElById("MainMenu.Channels.All")];
      } else if ('Cinema' == itemName) {
        items = [getElById("MainMenu.Cinema.Films"), getElById("MainMenu.Cinema.Kids")];
      }
      var navi = null;
      if (items) {
        navi = new Navigable();
        navi.setupNaviWithLine(items, 'h', true);
      }
      return navi;
    },

    getSubItemsRootEl: function(item) {
      return getElById('MainMenu.' + item + '.Subitems');
    },

    getItemRootEl: function(item) {
      return getElById('MainMenu.' + item);
    },

    getSubItemRootEl: function(item, subItem) {
      return getElById('MainMenu.' + item + '.' + subItem);
    }
  },

  getTypingChannelScreen: function(config) {
    if (!this.typingChannelScreen) {
      this.typingChannelScreen = this.getDialog(ScreenTypes.TYPE_CHANNEL_NO, config);
    }
    return this.typingChannelScreen;
  },

  getConfirmTokenCodeScreen: function(config) {
    if (!this.confirmTokenScreen) {
      this.confirmTokenScreen = this.getScreen(ScreenTypes.CONFIRM_TOKEN, $.extend(true, {rootEl:getElById("DialogWrapper"), mode: 'modal'}, config));
    }
    return this.confirmTokenScreen;
  },

  getNaviHelp: function() {
    if (!this.naviHelp) {
      this.naviHelp = new NavigationHelp(getElById("NavigationHelp"));
    }
    return this.naviHelp;
  },

  getRemoconFocusContainer: function() {
    return getElById("RemoconFocus");
  }
}
