/**
 *  config: PagedDisplay config + { rowLength: Int }
 */
function FilmsPagedDisplay(rootEl, config) {
  PagedDisplay.call(this, rootEl, config);
}

FilmsPagedDisplay.inheritsFrom(PagedDisplay);

/**
 * d - direction (ability to navigate from back)
 * fp - position (ability to fix row or column on page changed)
 */
// public
FilmsPagedDisplay.prototype.showPage = function(page, d, fp) {
  var p = page || 0;
  var startIndex = p * this.pageSize;
  if (startIndex < this.model.totalCount || 0 == p) {
    var self = this;
    this.model.get({ start: startIndex, count: this.pageSize, callback: { scope: self,
      fn: function(films) {
        $(self.rootEl).show();
        self.curPage = p;
        self.setupNavi(self.renderPage(films), {vShift: self.vShift, hShift: self.hShift});
        d && self.setCurEl(null, {dir:d, fixed: fp});
      }
    }});
    return true;
  }
  return false;
}

// public
FilmsPagedDisplay.prototype.updatePage = function(config, id) {
  if (config) {
    this.updateConfig(config);
  }
  var items = this.getItems(this.curPage * this.pageSize, this.pageSize);
  if (0 == items.length && this.curPage != 0) {
    this.curPage--;
    items = this.getItems(this.curPage * this.pageSize, this.pageSize);
  }
  this.setupNavi(this.renderPage(items));
  var c = this.navi.getAllItems().filter(function(el) {return el && el.info.queued_id == id;});
  var focusedEl = ( 0 < c.length ) ? c[0] : null;
  this.setCurEl(focusedEl);
}

// protected override
FilmsPagedDisplay.prototype.renderPage = function(items) {
  var films = [];
  if (items) {
    for (var i = 0; i < items.length; i++) {
      var p = this.getPosByPageOffset(i);
      if (!films[p.x]) {
        films[p.x] = [];
      }
      items[i].filmId = 'film' + i;
      films[p.x][p.y] = {info: items[i], naviId: items[i].filmId};
    }
  }

  var html = new EJS({url: 'templates/films.ejs'}).render({films: items,
    emptyMessage: Locale.t('Queue.EmptyFav')});
  widgetAPI.putInnerHTML(this.rootEl, html);
  return films;
}

// protected override
FilmsPagedDisplay.prototype.getPosByPageOffset = function(pageOffset) {
  var yy = Math.floor(pageOffset / this.rowLength);
  var xx = pageOffset - (this.rowLength * yy);
  return {x: xx, y: yy};
}

