function Navi(owner, naviTable, border, curPos) {
  this.owner = owner;
  if (naviTable && naviTable[0]) {
    this.naviTable = naviTable;
  } else {
    this.naviTable = [[]];
  }
  var b = border || {};
  this.border = this.createBorder(this.naviTable, border);
  this.setCurPos((curPos || this.defaultPos()), Navi.CREATION); // {x, y}
}

Navi.LEFT = {dx: -1, dy: 0};
Navi.RIGHT = {dx: 1, dy: 0};
Navi.UP = {dx: 0, dy: -1};
Navi.DOWN = {dx: 0, dy: 1};

Navi.CREATION = 'create';
Navi.MOVING = 'moving';
Navi.UPDATING = 'updating';
Navi.FOCUSING = 'focusing';
Navi.RESET = 'reset';

/**
 * moves current position at the direction vector if next position is not restricted by borders and there is an element at the position 
 * @param d = {dx, dy}, dx, dy in [-1, 0, 1]
 * @return selected element if position was moved, or false otherwise
 **/
// public
Navi.prototype.move = function(d) {
  var np = this.movedPos(d);
  var nEl = this.elAt(np);
  var moveEnabled = ( this.border.left <= np.x && np.x <= this.border.right  && this.border.top <= np.y && np.y <= this.border.bottom) ;
  this.setCurPos((moveEnabled && nEl ? np : this.curPos), Navi.MOVING, {dir: d});
  return moveEnabled && nEl;
}

// public
Navi.prototype.update = function(owner, naviTable, border, pos) {
  this.owner = owner;
  if (naviTable && naviTable[0]) {
    this.naviTable = naviTable;
  } else {
    this.naviTable = [[]];
  }
  this.border = this.createBorder(this.naviTable, border);
  this.setCurPos((pos || this.defaultPos()), Navi.UPDATING);
  return this;
}

/**
 * it sets current position as:
 *   1. position of el, if it is specified
 *   2. current position if saveCur==true and current position is defined
 *   3. position that is conf.pos, if it's specified
 *   4. position that calcs as shiftedPos, if conf.dir is specified
 *   5. default position at last
 *
 * conf: {
 *   pos: position to set
 *   dir: direction to shift (ability to navigate from back to front)
 *   fixed: position to start shift (ability to fix column or raw during paged navigation)
 *   saveCur: boolean, point to save current position
 *   withoutFocus: boolean
 * }
 */
// public
Navi.prototype.setCurEl = function(el, conf) {
  var c = conf || {};
  var p = null;
  if (el) {
    p = this.getPosByEl(el);
  } else if (c.saveCur && this.curPos) {
    p = this.curPos;
  } else {
    p = (c.pos || (c.dir && this.shiftedPos(c.dir, c.fixed)) || this.defaultPos());
  }
  if (p) {
    this.setCurPos(p, Navi.FOCUSING, {dir: c.dir, fixed: c.fixed, saveCur: c.saveCur, withoutFocus: c.withoutFocus});
  }
  return this.curEl();
}

// public
Navi.prototype.curEl = function() {
  return this.elAt(this.curPos);
}

// public
Navi.prototype.elAt = function(pos) {
  return pos && this.naviTable[pos.x] && this.naviTable[pos.x][pos.y];
}

// public
Navi.prototype.defaultPos = function() {
  return {x: this.border.left, y: this.border.top};
}

// public
Navi.prototype.currentPos = function() {
  return this.curPos ? {x: this.curPos.x, y: this.curPos.y} : null;
}

// public (helper for Navigable)
// TODO: package m.b.?
Navi.prototype.reset = function() {
  this.setCurPos(null, Navi.RESET);
}

// public (helper for Navigable)
Navi.prototype.getAllItems = function() {
  var items = [];
  for (var i = 0; i < this.naviTable.length; i++) {
    for (var j = 0; j < this.naviTable[0].length; j++) {
      items.push(this.naviTable[i][j]);
    }
  }
  return items;
}

// internal
Navi.prototype.isEmpty = function() {
  var empty = true;
  for (var i = 0; i < this.naviTable.length; i++) {
    if (0 < this.naviTable[i].length) {
      empty = false;
    }
  }
  return empty;
}

/////////////////////////////////////////////////////////////////////////////

// private
Navi.prototype.setCurPos = function(p, reason, conf) {
  var prevP = this.curPos;
  this.curPos = p;
  this.owner.onCursorSet.call(this.owner, reason, conf);
  if ( !prevP || !p || (prevP.x != this.curPos.x) || (prevP.y != this.curPos.y)) {
    this.owner.onCursorChanged.call(this.owner, reason, conf);
  }
}

// private
Navi.prototype.movedPos = function(d) {
  var b = this.border;

  var nx = this.curPos.x + d.dx;
  var hl = b.right - b.left + 1; // horizontal length
  nx = b.hShift ? ((nx + hl- b.left) % hl) + b.left : nx;

  var ny = this.curPos.y + d.dy;
  var vl = b.bottom - b.top + 1; // vertical length
  ny = b.vShift ? ((ny + vl - b.top) % vl) + b.top : ny;
  
  return {x: nx, y: ny};
}

// private
Navi.prototype.shiftedPos = function(d, p) {
  var pos = p || this.defaultPos();
  pos.x = (0 == d.dx) ? pos.x : (d.dx < 0) ? this.border.right : this.border.left;
  pos.y = (0 == d.dy) ? pos.y : (d.dy < 0) ? this.border.bottom : this.border.top;
  return pos;
}

// private
Navi.prototype.getPosByEl = function(el) {
  for(var i = this.border.left; i <= this.border.right; i++) {
    for(var j = this.border.top; j <= this.border.bottom; j++) {
      if(this.elAt({x: i, y: j}) == el) {
        return {x: i, y: j};
      }
    }
  }

  return null;
}

// private
Navi.prototype.createBorder = function(table, originalBorder) {
  var b = originalBorder || {};
  return {top: (isNum(b.top) && b.top || 0), bottom: (isNum(b.bottom) && b.bottom || Math.max(0, table[0].length - 1))
  , left: (isNum(b.left) && b.left || 0), right: (isNum(b.right) && b.right || Math.max(0, table.length - 1))
  , vShift: b.vShift, hShift: b.hShift};
}

