function ChannelsScreen(config) {
  Screen.call(this, config);
  this.subTypes = ['All', 'Favorites'];
  this.model = ModelsFactory.getChannels();

  // TODO: поправить верстку для navigation help и добавить эту строчку (сейчас не преусмотрено для широких иконок)
  //this.naviButtons = this.naviButtons.concat({name: NavigationButtons.DIGITS, text: Locale.t('HB.TypingChannel')});
}

ChannelsScreen.inheritsFrom(Screen);

// public override
ChannelsScreen.prototype.getNaviButtons = function() {
  var nb = this.naviButtons.concat({name: NavigationButtons.GREEN, text: Locale.t('HB.Reload')});
  if (this.channelsPagedDisplay && this.channelsPagedDisplay.hoveredItem) {
    nb = nb.concat({name: NavigationButtons.YELLOW, text: this.channelsPagedDisplay.hoveredItem.info.favourited ? Locale.t('HB.RemoveChannelFromFav') : Locale.t('HB.AddChannelToFav')});
  }
  return nb;
}


ChannelsScreen.prototype.keyHandler = function(keyEvent) {
  switch(keyEvent.keyCode) {
    case tvKey.KEY_0:
    case tvKey.KEY_1:
    case tvKey.KEY_2:
    case tvKey.KEY_3:
    case tvKey.KEY_4:
    case tvKey.KEY_5:
    case tvKey.KEY_6:
    case tvKey.KEY_7:
    case tvKey.KEY_8:
    case tvKey.KEY_9:
      WidgetsFactory.getTypingChannelScreen().show({keyEvent: keyEvent, okCallback: {fn: this.onChannelTyped, scope: this} });
      break;
    case tvKey.KEY_YELLOW:
      var self = this;
      if (this.channelsPagedDisplay.hoveredItem) {
        var abbr = this.channelsPagedDisplay.hoveredItem.info.abbr;
        this.model.toggleFavourited({ channel: self.channelsPagedDisplay.hoveredItem.info, callback: { scope: self,
          fn: function(success, channels) {
            self.channels = channels;
            self.channelsPagedDisplay.updatePage({source: self.getActualChannels()}, abbr);
            WidgetsFactory.getNaviHelp().updateNavi(self.getNaviButtons());
          }
        }});
      }
      break;
    case tvKey.KEY_GREEN:
      var self = this;
      var channel = (self.channelsPagedDisplay.hoveredItem && self.channelsPagedDisplay.hoveredItem.info);
      this.model.get({update: true, callback: {scope: self,
        fn: function(channels) {
          if (channels) {
            self.channels = channels;
            self.processSubType(self.subType, channel);
          }
        }
      }});
      break;
    default:
      break;
  }

  WidgetsFactory.getNaviHelp().updateNavi(this.getNaviButtons());
};

// public override
ChannelsScreen.prototype.processSubType = function(subType, channel) {
  this.subType = subType || this.subType;
  this.displayChannels(this.getActualChannels(), channel);
}


// protected override
ChannelsScreen.prototype.fill = function(config) {
  var self = this;
  this.model.get({callback: {scope: self,
    fn: function(channels) {
      if (channels) {
        self.channels = channels;
        self.processSubType(config.subType, config.channel);
        self.onFillComplete(config.focused);
      }
    }
  }});
};

// private
ChannelsScreen.prototype.displayChannels = function(channels, currentChannel) {
  this.channelsPagedDisplay = WidgetsFactory.channelsScreen.getChannelsPagedDisplay({
    rootEl: this.rootEl,
    source: channels});

  var self = this;
  $(this.channelsPagedDisplay).bind(Events.NAVI_SELECTION_SET,
    function(event, selectedContentItem) { self.onChannelSelected(selectedContentItem);} );

  this.setupNavi([[this.channelsPagedDisplay]]);
  this.setCurEl();
  if (currentChannel) {
    this.channelsPagedDisplay.showPageWithEl({scope: this, fn: function(ch) {
      return ch.abbr == currentChannel.abbr;
    }}, true);
  } else {
    this.channelsPagedDisplay.showPage();
  }
}

// private
ChannelsScreen.prototype.getActualChannels = function() {
  var c = [];
  if ('All' == this.subType) {
    c = this.channels;
  } else if ('Favorites' == this.subType) {
    c = this.getFavouriteChannels();
  }
  return c;
}

// private
ChannelsScreen.prototype.onChannelSelected = function(selectedChannel) {
  if (selectedChannel) {
    this.showLive(selectedChannel.info);
  }
};

// private
// callback to TypingChannelScreen
ChannelsScreen.prototype.onChannelTyped = function(channelId) {
  WidgetsFactory.getTypingChannelScreen().hide();
  if (channelId) {
    var c = this.channels.filter(function(el) {
      return channelId == el.button;
    });
    if (c[0]) {
      this.showLive(c[0]);
    }
  }
};

// private
ChannelsScreen.prototype.getFavouriteChannels = function() {
  return this.channels.filter(function(el){
    return el.favourited;
  });
}

// private
ChannelsScreen.prototype.showLive = function(channel) {
  this.childScreen = WidgetsFactory.getScreen(ScreenTypes.LIVE, {parentScreen: this});
  this.childScreen.show({channel: channel});
};

