function LiveScreen(config) {
  Screen.call(this, config);
  this.model =ModelsFactory.getTelecasts();
}

LiveScreen.inheritsFrom(Screen);

// public override
LiveScreen.prototype.getNaviButtons = function() {
  var nb = this.naviButtons.concat({name: NavigationButtons.GREEN, text: Locale.t('HB.Reload')});
  if (this.channel) {
    nb = nb.concat({name: NavigationButtons.YELLOW, text: this.channel.favourited ? Locale.t('HB.RemoveChannelFromFav') : Locale.t('HB.AddChannelToFav')});
  }
  return nb;
}

// internal override
LiveScreen.prototype.keyHandler = function(keyEvent) {
  switch(keyEvent.keyCode) {
    case tvKey.KEY_YELLOW:
      this.toggleFavorite();
      break;
    case tvKey.KEY_GREEN:
      var self = this;
      this.model.get({ update: true, channel: this.channel, callback: {scope: self, fn:
        function(telecasts) {
          if (telecasts) {
            self.channelIssues = telecasts;
            self.display(self.channel, self.channelIssues);
          }
        }
      }});
      break;
    default:
      break;
  }

}

//////////////////////////////////////////////////////////////////

// protected override
LiveScreen.prototype.fill = function(config) {
  this.channel = config.channel || this.channel;
  var self = this;
  if (config.back) {
    this.display(self.channel, this.channelIssues, config.videoItem);
    this.onFillComplete(config.focused);
  } else {
    this.model.get({ channel: this.channel, callback: {scope: self, fn:
      function(telecasts) {
        if (telecasts) {
          self.channelIssues = telecasts;
          self.display(self.channel, self.channelIssues);
          self.onFillComplete(config.focused);
        }
      }
    }});
  }
}

// protected override
LiveScreen.prototype.getBackConfig = function() {
  return {channel: this.channel};
}

//////////////////////////////////////////////////////////////////////

// private
LiveScreen.prototype.display = function(channel, channelIssues, backVideoItem) {
  var self = this;
  var html = new EJS({url: 'templates/live.ejs'}).render(channel);
  widgetAPI.putInnerHTML(this.rootEl, html);

  this.updateFavouritedStatus(channel.favourited);

  this.channelMenu = WidgetsFactory.liveScreen.getChannelMenuNavi();
  $(this.channelMenu).bind(Events.NAVI_SELECTION_SET,
    function(e, selectedItem) { self.onMenuSelectionChanged(selectedItem);
  });

  this.issuesPagedDisplay = WidgetsFactory.liveScreen.getIssuesPagedDisplay({
    source: channelIssues});
  $(this.issuesPagedDisplay).bind(Events.NAVI_SELECTION_SET,
    function(e, selectedIssue) {self.onIssueSelectionChanged(selectedIssue);} );

  this.setupNaviWithLine([this.channelMenu, this.issuesPagedDisplay], 'h', true);
  var getFocusedItem = null;
  if (backVideoItem && backVideoItem.beginning) {
    getFocusedItem = function(telecast) {
      return (backVideoItem.beginning == telecast.beginning);
    }
  } else {
    getFocusedItem = function(telecast) {
      return telecast.current;
    }
  }
  this.issuesPagedDisplay.showPageWithEl({scope: this, fn: getFocusedItem}, true);
  this.setCurEl(this.issuesPagedDisplay, {saveCur: true});
};

// private
LiveScreen.prototype.onIssueSelectionChanged = function(item) {
  if (item.info.available) {
    this.tryShowVideo(item.info);
  }
};

// private (callback)
LiveScreen.prototype.onMenuSelectionChanged = function(selectedItem) {
   var f = getElLastName(selectedItem);
   this[f] && this[f]();
}

// private (callback)
LiveScreen.prototype.showLive = function() {
  this.channel.streamUrls = this.channel.liveUrls;
  this.tryShowVideo(this.channel);
}

// private (callback)
LiveScreen.prototype.showTimeshift = function() {
  this.channel.streamUrls = this.channel.timeshiftUrls;
  this.tryShowVideo(this.channel);
}

// private (callback)
LiveScreen.prototype.onVideoComplete = function(video) {
  var self = this;
  var nextVideo = this.getNextVideo(video);
  var note = WidgetsFactory.getNotificationScreen();
  var msg = (nextVideo.beginning) ? Locale.t('Live.CompleteDialog.ShowNextAirtime') : Locale.t('Live.CompleteDialog.ShowLive');
  note.show({message: msg, buttons: [
    { label: Locale.t('Live.CompleteDialog.Next'), callback: { fn:
        function() {
          note.hide();
          WidgetsFactory.getPlayerScreen().setVideoItem(nextVideo, true);
        },
      scope:self }
    },
    { label: Locale.t('Live.CompleteDialog.Back'), callback: { fn:
        function() {
          note.hide();
          WidgetsFactory.getPlayerScreen().return();
        },
      scope: self }
    }]
  });
}

// private
LiveScreen.prototype.getNextVideo = function(currentVideo) {
  var video;
  var i = 0;
  while (!video && i < this.channelIssues.length) {
    if (currentVideo.beginning < this.channelIssues[i].beginning) {
      video = this.channelIssues[i];
    }
    i++;
  }

  if (!(video && video.available)) {
    this.channel.streamUrls = this.channel.liveUrls;
    video = this.channel;
  }
  return video;
}

// private (callback)
LiveScreen.prototype.toggleFavorite = function() {
  var self = this;
  ModelsFactory.getChannels().toggleFavourited({channel: self.channel, callback: {scope:self,
    fn: function(success, channels) {
      if (success) {
        self.channel.favourited = !self.channel.favourited;
        self.updateFavouritedStatus(self.channel.favourited);
      }
    }
  }});
}

// private
LiveScreen.prototype.updateFavouritedStatus = function(favourited) {
  WidgetsFactory.liveScreen.updateFavouritedStatus(favourited);
  WidgetsFactory.getNaviHelp().updateNavi(this.getNaviButtons());
}

// private
LiveScreen.prototype.tryShowVideo = function(videoItem) {
  var hasAccess = videoItem.videostatus == 'full_access' && videoItem.streamUrls && videoItem.streamUrls[0];
  if (hasAccess) {
    var self = this;
    this.childScreen = WidgetsFactory.getPlayerScreen({parentScreen: this});
    this.childScreen.show({videoItem: videoItem, play: true,
      completeCallback: {fn: self.onVideoComplete, scope: self}
    });
  } else {
    WidgetsFactory.getNotificationScreen().show({message: Locale.t('Notification.VideoAccess')});
  }
}

