/**
 * screen config + {
 *   player,
 *   audio
 * }
 */

function PlayerScreen(config) {
  Screen.call(this, config);
  this.addAudioListeners();
  this.addPlayerListeners();

  this.BUFFER_BAR_WIDTH = 322;
  this.VOLUME_BAR_WIDTH = 79;

  this.SEEK_SPEEDS = [15, 1*60, 3*60];
  this.SEEK_FW = 'ff';
  this.SEEK_BW = 'rw';
  this.SEEK_INTERVAL = 1 * 1000;

  this.HISTORY_INTERVAL = 5 * 60 * 1000;
  this.PAUSE_LIVE_INTERVAL = 4 * 60 * 60 * 1000;
}

PlayerScreen.inheritsFrom(Screen);

// protected override
PlayerScreen.prototype.fill = function(config) {
  if (config.videoItem) {
    var html = new EJS({url: 'templates/player.ejs'}).render(config.description);
    widgetAPI.putInnerHTML(this.rootEl, html);
    this.initComponents();
    this.updateTime(0, 0);
    this.updateHidePanelTimer(false);
    this.setVideoItem(config.videoItem, config.play);
    this.completeCallback = config.completeCallback;
    this.onFillComplete(config.focused);
  }
}

// protected override
PlayerScreen.prototype.beforeHide = function() {
  if (this.videoItem && this.videoItem.airtime_id) {
    this.saveHistory();
  }
  this.videoItem = null;
  this.reset();
  this.player.stop();
  this.player.setUrl(null);
  this.player.setScreenMode(this.player.WM);
  this.updateTime(0, 0);
  this.seekState = null;
  clearTimeout(this.hidePanelTimer);
  clearInterval(this.seekTimer);
  clearInterval(this.historyTimer);
  clearInterval(this.pauseLiveTimer);
}

// protected override
PlayerScreen.prototype.getBackConfig = function() {
  return {videoItem: this.videoItem};
}

// private
PlayerScreen.prototype.setVideoItem = function(videoItem, play) {
alert("PlayerScreen.setVideoItem() " + objToString(videoItem));
  this.videoItem = videoItem;
  this.updateTime(this.curTime, this.duration);
  this.updateVolume(this.audio.getVolume());
  this.updateMute(this.audio.getMute());
  $(WidgetsFactory.playerDisplay.title()).text(videoItem.title || '');
  this.player.setUrl(this.videoItem);
  if (play || this.player.isState(this.player.PLAYING)) {
    this.streamControlEnabled = false;
    this.setWaitingStatus();
    this.player.play();
    this.player.setScreenMode(this.player.FS);
  } else {
    this.streamControlEnabled = true;
    $(this.status).attr('class', 'player-status player-status_pause');
    $(this.status).show();
  }
}


// internal override
PlayerScreen.prototype.keyHandler = function(keyEvent) {
  var keyCode = event.keyCode;

  this.playerKeyHandler(keyCode);
  this.audioKeyHandler(keyCode);

  this.updateHidePanelTimer(!(this.seekState || this.waitingStatus));
}

// protected override
PlayerScreen.prototype.selectItem = function(item) {
  this.parent.selectItem.call(this, item);
  item && item.onSelect && item.onSelect.call(this);
}

// private
PlayerScreen.prototype.updateVolume = function(volume) {
  $(WidgetsFactory.playerDisplay.volumeStatus()).css('width', Math.round(this.VOLUME_BAR_WIDTH * volume / 100) + 'px');
}

// private
PlayerScreen.prototype.updateMute = function(muted) {
  muted ? this.btMute.setState(this.btMute.MUTED)
      : this.btMute.setState(this.btMute.VOLUMED);
}

// private
PlayerScreen.prototype.onStatusChanged = function(status) {
  alert("PlayerScreen.onStatusChanged " + status);
  if (status == this.player.PLAYING) {
    this.btTogglePlay.setState(this.btTogglePlay.PLAYING);
    $(this.status).hide();
    if (this.isLive) {
      this.updatePauseLiveTimer(true);
    }
  } else {
    this.btTogglePlay.setState(this.btTogglePlay.STOPPED);

    $(this.status).attr('class', 'player-status player-status_pause');
    $(this.status).show();
    this.updatePauseLiveTimer(false);
  }
  if (this.videoItem && this.videoItem.airtime_id) {
    this.updateHistoryTimer(this.player.PLAYING == status);
  }
  this.waitingStatus = false;
}

// private
PlayerScreen.prototype.onBufferChanged = function(type, buffer) {
  if (Events.PLAYER_BUFFERING_COMPLETE == type) {
    this.updateHidePanelTimer(true);
    $(this.status).hide();
  } else if (Events.PLAYER_BUFFERING_START == type) {
    this.updateHidePanelTimer(false);
    $(this.status).attr('class', 'player-status player-status_buffering');
    $(this.status).show();
  }
  $(WidgetsFactory.playerDisplay.bufferingBar()).css('width', Math.round(this.BUFFER_BAR_WIDTH * (buffer || 0)/100) + 'px');
  alert("buffering " + type + " " + buffer);
}

// private
PlayerScreen.prototype.onSeek = function() {
  this.setWaitingStatus();
}

// private
PlayerScreen.prototype.onCurrentTime = function(curTime, maxTime) {
  this.updateTime(curTime, maxTime || this.duration);
}

// private
PlayerScreen.prototype.onStreamInfo = function(info) {
  alert("!!!!!!! ON STREAM INFO " + objToString(info));
  this.updateLiveDependsComponents(info);
  this.updateTime(this.curTime, info.duration);
  this.streamControlEnabled = true;
}

// private
PlayerScreen.prototype.onComplete = function() {
  alert("PlayerScreen.onComplete " + this.completeCallback);
  this.updatePauseLiveTimer(false);
  if (this.completeCallback) {
    this.completeCallback.fn.call(this.completeCallback.scope, this.videoItem);
  }
}

// private
PlayerScreen.prototype.updateLiveDependsComponents = function(info) {
  if (info.seekable) {
    $(this.btBW).show();
    $(this.btFW).show();
    this.setupNaviWithLine([this.btBack, this.btBW, this.btTogglePlay, this.btFW, this.btMute], 'h', true);

  } else {
    $(this.btBW).hide();
    $(this.btFW).hide();
    this.setupNaviWithLine([this.btBack, this.btTogglePlay, this.btMute], 'h', true);
  }
  this.isLive = info.isLive;
  this.seekable = info.seekable;
  this.updatePauseLiveTimer(this.isLive && this.player.isState(this.player.PLAYING));
  this.progressDisplay && this.progressDisplay.deactivate();
  this.progressDisplay = WidgetsFactory.playerDisplay.getProgressDisplay(info.duration);
  this.progressDisplay.activate(this.videoItem);
  this.progressDisplay.updateTime(this.curTime, this.duration);
  this.setCurEl(this.btTogglePlay);
}

// private
PlayerScreen.prototype.updateTime = function(curTime, duration) {
  this.curTime = curTime || 0;
  this.duration = duration;
  this.progressDisplay && this.progressDisplay.updateTime(this.curTime, this.duration);
}

// private
PlayerScreen.prototype.initComponents = function() {
  var self = this;

  this.btTogglePlay = WidgetsFactory.playerDisplay.btTogglePlay();
  this.btTogglePlay.onSelect = function() {
    if (self.seekState) {
      self.handlePlayKey();
    } else {
      self.playerEnabled() && self.player.togglePlay();
    }
  };

  this.btBW = WidgetsFactory.playerDisplay.btBW();
  this.btBW.onSelect = function() { self.playerEnabled() && self.handleSeekKey(self.SEEK_BW) };

  this.btFW = WidgetsFactory.playerDisplay.btFW();
  this.btFW.onSelect = function() { self.playerEnabled() && self.handleSeekKey(self.SEEK_FW) };

  this.btBack = WidgetsFactory.playerDisplay.btBack();
  this.btBack.onSelect = function() { self.return(); };

  this.btMute = WidgetsFactory.playerDisplay.btToggleMute();
  this.btMute.onSelect = function() { self.audio.toggleMute(); };

  this.status = WidgetsFactory.playerDisplay.status();

  this.progressDisplay = WidgetsFactory.playerDisplay.getStandartProgressDisplay();
}

// private
PlayerScreen.prototype.addPlayerListeners = function() {
  //TODO: add error callbacks
  var self = this;
  $(this.player).bind(Events.PLAYER_STATE_CHANGED
    , function(e){self.onStatusChanged(e.info);});
  $(this.player).bind([Events.PLAYER_BUFFERING_START, Events.PLAYER_BUFFERING_PROGRESS, Events.PLAYER_BUFFERING_COMPLETE].join(' ')
    , function(e){self.onBufferChanged(e.type, e.info);});
  $(this.player).bind(Events.PLAYER_SEEK
    , function(e){self.onSeek();})
  $(this.player).bind(Events.PLAYER_STREAM_INFO_READY
    , function(e){self.onStreamInfo(e.info);});
  $(this.player).bind(Events.PLAYER_CURRENT_TIME
    , function(e, curTime, maxTime){self.onCurrentTime(curTime, maxTime)});
  $(this.player).bind(Events.PLAYER_COMPLETE
    , function(e){self.onComplete();});
  $(this.player).bind(Events.PLAYER_ERROR,
    function(e, type){
      if ('disconnect' == type) {
        $(self.status).hide();
        if (self.seekable) {
          var note = WidgetsFactory.getNotificationScreen();
          note.show({message: Locale.t('Notification.Disconnected'),
            buttons:[{label:Locale.t('Notification.Accept'), callback:{scope: self,
              fn: function() {
                note.hide();
                self.reconnect();
              }
            }}]});
        } else {
          self.reconnect();
        }
      }
    }
  );
}

// private
PlayerScreen.prototype.addAudioListeners = function() {
  var self = this;
  $(this.audio).bind(Events.VOLUME_CHANGED
    , function(e){self.updateVolume(e.info)});
  $(this.audio).bind(Events.MUTE_CHANGED
    ,function(e){self.updateMute(e.info)});
}

// private
PlayerScreen.prototype.playerKeyHandler = function(keyCode) {
  if (!this.playerEnabled()) {
    return;
  }

  switch(keyCode)
    {
      case tvKey.KEY_PLAY:
        this.handlePlayKey();
        break;
      case tvKey.KEY_STOP:
      case tvKey.KEY_PAUSE:
        this.player.pause();
        break;
      case tvKey.KEY_FF:
        this.handleSeekKey(this.SEEK_FW);
        break;
      case tvKey.KEY_RW:
        this.handleSeekKey(this.SEEK_BW);
        break;
      default :
        break;
    }
}

// private
PlayerScreen.prototype.audioKeyHandler = function(keyCode) {
  switch(keyCode)
  {
    case tvKey.KEY_VOL_UP:
    case tvKey.KEY_PANEL_VOL_UP:
      this.audio.setVolumeWithDelta(this.audio.UP);
      break;
    case tvKey.KEY_VOL_DOWN:
    case tvKey.KEY_PANEL_VOL_DOWN:
      this.audio.setVolumeWithDelta(this.audio.DOWN);
      break;
    case tvKey.KEY_MUTE:
      this.audio.toggleMute();
      break;
    default :
      break;
  }
}

// private
PlayerScreen.prototype.playerEnabled = function() {
  return (this.videoItem != null && this.streamControlEnabled);
}

// private
PlayerScreen.prototype.handleSeekKey = function(dir) {
  if (!this.seekable) {
    return;
  }
  if (this.seekState && this.seekState.freezed) {
    return;
  }
  if (this.seekState) {
    if (this.seekState.dir == dir) {
      this.seekState.speed = (this.seekState.speed + 1) % this.SEEK_SPEEDS.length;
    } else {
      this.seekState.speed = 0;
      this.seekState.dir = dir;
    }
  } else {
    this.timeBeforeSeek = this.curTime;
    this.seekState = {dir: dir, speed: 0};
    this.player.isState(this.player.PLAYING) && this.player.pause();
    this.updateHidePanelTimer(false);
    this.updateSeekTimer(true);
  }
  $(this.status).attr('class', 'player-status player-status_' + this.seekState.dir + ((0 == this.seekState.speed) ? '' : (this.seekState.speed + 1)));
  $(this.status).show();
}

// private
PlayerScreen.prototype.handlePlayKey = function() {
  if (!this.playerEnabled() || (this.seekState && this.seekState.freezed)) {
    return;
  }
  // варианты выхода из режима seek (player в состоянии pause):
  // 1. делать player.play(), а потом player.seek(). тогда после нажатия кнопки play проигрывается небольшое количество видео (заметно не больше секунды).
  // 2. оставлять плеер в состоянии pause до события buffer complete и после этого делать player.play(). тогда сначала слышен звук, а стоящая картинка сменяется виеорядом секунд через 5
  // используем вариант 2.
  // в обоих случаях эпизодически проявляется повторная буферизация
  if (this.seekState && !this.seekState.freezed) {
    this.updateSeekTimer(false);
    var seekOffset = Math.round((this.curTime - this.timeBeforeSeek)/1000);
    var self = this;
    var onSeekReadyCallback = { scope: self, fn: function() {
      self.player.play();
      self.seekState = null;
    }};
    this.seekState.freezed = true;
    if (0 < seekOffset) {
      this.player.fw(Math.abs(seekOffset), onSeekReadyCallback);
    } else {
      this.player.bw(Math.abs(seekOffset), onSeekReadyCallback);
    }
  } else if (!this.player.isState(this.player.PLAYING)) {
    this.player.play();
  }
}
// private
PlayerScreen.prototype.getSeekTime = function() {
  var t = 0;
  if (this.seekState) {
    var d = this.SEEK_SPEEDS[this.seekState.speed] * 1000;
    if (this.SEEK_FW == this.seekState.dir) {
      t = parseInt(this.curTime) + d;
      t = this.duration ? Math.min(t, this.duration) : t;
    } else if (this.SEEK_BW == this.seekState.dir) {
      t = Math.max(parseInt(this.curTime) - d, 0);
    }
  }
  return t;
}

// private
PlayerScreen.prototype.saveHistory = function() {
  AppFactory.getRequestsFactory().saveHistoty(this.videoItem.airtime_id, Math.floor(this.curTime / 1000));
}

// private
PlayerScreen.prototype.reconnect = function() {
   this.setWaitingStatus();
   this.player.reconnect();
}

// private
PlayerScreen.prototype.setWaitingStatus = function() {
  this.updateHidePanelTimer(false);
  $(this.status).attr('class', 'player-status player-status_seeking');
  $(this.status).show();
  this.waitingStatus = true;
}

// private
PlayerScreen.prototype.updateSeekTimer = function(on) {
  clearInterval(this.seekTimer);
  if (on) {
    var self = this;
    this.seekTimer = setInterval(function() {self.updateTime(self.getSeekTime(), self.duration)}, this.SEEK_INTERVAL);
  }
}

// private
PlayerScreen.prototype.updateHidePanelTimer = function(on) {
  $(this.rootEl).show();
  clearTimeout(this.hidePanelTimer);
  if (on) {
    var self = this;
    this.hidePanelTimer = setTimeout(function() {
      self.player.isState(self.player.PLAYING) && $(self.rootEl).hide();
    }, 3*1000);
  }
}

// private
PlayerScreen.prototype.updateHistoryTimer = function(on) {
  clearInterval(this.historyTimer);
  if (on) {
    var self = this;
    this.historyTimer = setInterval(function() {
      self.saveHistory();
    }, this.HISTORY_INTERVAL);
  }
}

// private
PlayerScreen.prototype.updatePauseLiveTimer = function(on) {
  clearTimeout(this.pauseLiveTimer);
  if (on) {
    var self = this;
    this.pauseLiveTimer = setTimeout(function() {
      if (self.player.isState(self.player.PLAYING)) {
        self.player.pause();
        self.updatePauseLiveTimer(false);
        self.updateHidePanelTimer(false);
      }
    }, this.PAUSE_LIVE_INTERVAL);
  }
}


