/**
 * config = {
 *   shownCallback,
 *   hideCallback,
 *   parentScreen,
 *   rootEl,
 *   mode: ['screen', 'dialog', 'player'],
 */
function Screen(config) {
  this.updateConfig(config);
}

Screen.inheritsFrom(Navigable);

// public
Screen.prototype.updateConfig = function(config) {
  $.extend(true, this, config);
  this.naviButtons = [{name: NavigationButtons.BACK, text: Locale.t('HB.Return')}];
  if ('screen' == this.mode) {
    this.naviButtons.push({name: NavigationButtons.TOOLS, text: Locale.t('HB.ToMM')});
  }
}

// public
Screen.prototype.getNaviButtons = function() {
  return this.naviButtons;
}

/**
 * config = {
 *   focused: Boolean, default TRUE
 * }
 */
// public
Screen.prototype.show = function(config) {
  if (this.dialogId) {
    Screen.currentDialogId = this.dialogId;
  }
  this.fill($.extend(true, {focused: true}, config));
}

// public
Screen.prototype.hide = function(config) {
  var c = config || {};
  if ('dialog' == this.mode) {
    if( !(c.force || Screen.currentDialogId <= this.dialogId)) {
      return;
    }
  }
  Screen.currentDialogId = null;
  this.childScreen && this.childScreen.hide();
  this.beforeHide();
  this.blur();
  $(this.rootEl).empty();
  if (this.hideCallback) {
    this.hideCallback.fn.call(this.hideCallback.scope, this);
  }
}

// public
Screen.prototype.focus = function() {
  // TODO: is it useable? m.b., remove it
  this.setCurEl(null, {saveCur: true});
  // TODO: set focus skin
}

// public
Screen.prototype.return = function() {
  var backConfig = this.getBackConfig();
  this.hide();
  this.parentScreen && this.parentScreen.show($.extend({back: true}, backConfig));
}

// public
Screen.prototype.hasParent = function() {
  return (this.parentScreen != null);
}

// public
Screen.prototype.hasSubType = function(subType) {
  return this.subTypes && arrContains(this.subTypes, subType);
}

// public to override
Screen.prototype.processSubType = function(subType) {
}

///////////////////////////////////////////////

// internal
Screen.prototype.keyHandler = function(keyEvent) {
}

////////////////////////////////////////////////

// protected
Screen.prototype.blur = function() {
  // TODO: set blur skin
  this.reset();
}

// protected
Screen.prototype.fill = function(config) {
  this.onFillComplete(config.focused);
}

// protected
Screen.prototype.onFillComplete = function(focused) {
  if (focused) {
    this.focus();
  }
  if (this.shownCallback) {
   this.shownCallback.fn.call(this.shownCallback.scope, this, focused);
  }
}

// protected
Screen.prototype.beforeHide = function() {

}

// protected
Screen.prototype.getBackConfig = function() {
  return null;
}

////////////////////////////////////////////////
