function TypingChannelScreen(config) {
  Screen.call(this, config);

  var self = this;
  $(this).bind(Events.NAVI_SELECTION_SET
    , function(event, selectedBt) {
      if (selectedBt.toLowerCase() == "ok") {
        if (self.okCallback) {
          self.okCallback.fn.call(self.okCallback.scope, self.getChannelNum());
        } else {
          self.hide();
        }
      } else if (selectedBt.toLowerCase() == "cancel"){
        if (self.cancelCallback) {
          self.cancelCallback.fn.call(self.cancelCallback.scope);
        } else {
          self.hide();
        }
      }
  });

}

TypingChannelScreen.inheritsFrom(Screen);

// public override
TypingChannelScreen.prototype.return = function() {
  this.hide();
  this.parentScreen && this.parentScreen.focus();
}

// internal override
TypingChannelScreen.prototype.keyHandler = function(keyEvent) {
  this.handleBackspace(keyEvent);
  this.handleDigitKey(keyEvent);
  this.updateDigits();
}

/**
 * config {
 *   channels: Array,
 *   okCallback,
 *   cancelCallback,
 *   keyEvent
 */
// protected override
TypingChannelScreen.prototype.fill = function(config) {
  $.extend(true, this, config);
  this.digits = [0, 0, 0];
  this.typedDigitsCount = 0;
  var html = new EJS({url: 'templates/typingChannel.ejs'}).render();
  widgetAPI.putInnerHTML(this.rootEl, html);
  this.setupNaviWithLine([getElById('TypingChannelScreen.Buttons.Ok'), getElById('TypingChannelScreen.Buttons.Cancel')], 'h', true);
  this.keyHandler(this.keyEvent);
  this.onFillComplete(config.focused);
}

// protected override
TypingChannelScreen.prototype.beforeHide = function() {
  this.digits = null;
  this.okCallback = null;
  this.cancelCallback = null;
  this.keyEvent = null;
}

// protected override
// TODO: copy pasted from MM. Extract it to Utils
TypingChannelScreen.prototype.getSelection = function() {
  return getElLastName(this.selectedItem);
}

// private
TypingChannelScreen.prototype.updateDigits = function() {
  this.digits && $(getElById("TypingChannelScreen.Digits")).text(this.digits.join(''));
}

// private
TypingChannelScreen.prototype.handleBackspace = function(keyEvent) {
  if (keyEvent.keyCode == tvKey.KEY_RW) {
    this.typedDigitsCount--;
    this.digits.pop();
    this.digits.unshift(0);
  }
}

// private
TypingChannelScreen.prototype.handleDigitKey = function(keyEvent) {
  var d = this.getDigit(keyEvent.keyCode);
  if (!isNaN(d)) {
    if (this.typedDigitsCount < Consts.MAX_CHANNEL_NUM_DIGITS) {
      this.typedDigitsCount++;
      this.digits.shift();
      this.digits.push(d);
    }
  }
}

// private
TypingChannelScreen.prototype.getDigit = function(keyCode) {
  var d = NaN;
  switch(keyCode) {
    case tvKey.KEY_0:
      d = 0;
      break;
    case tvKey.KEY_1:
      d = 1;
      break;
    case tvKey.KEY_2:
      d = 2;
      break;
    case tvKey.KEY_3:
      d = 3;
      break;
    case tvKey.KEY_4:
      d = 4;
      break;
    case tvKey.KEY_5:
      d = 5;
      break;
    case tvKey.KEY_6:
      d = 6;
      break;
    case tvKey.KEY_7:
      d = 7;
      break;
    case tvKey.KEY_8:
      d = 8;
      break;
    case tvKey.KEY_9:
      d = 9;
      break;
    default:
      break;
  }

  return d;
}

// private
TypingChannelScreen.prototype.getChannelNum = function() {
  var numStr = this.digits.join('');
  return +numStr;
}
